/*  
 *
 *  (c) 1999 Norm Bell
 *  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


// cdgpacker.cpp : Defines the entry point for the DLL application.
//
// permission is granted for free use this demo software to develop 
// applications as long as the applications are given away
// for free and are not sold.

#include "cdgpacker.h"
#include "cdgdef.h"

/*
// this routine would be used if this file was being
// built into a single dll - this may happen later....
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}
*/


/******************************************************************************
 *
 * Clear CDG Packet
 *
 *    This function takes an existing CDG Packet and clears the data area
 *    to zero utiliziing a 0x3F mask. The high 2 bbits are left as is.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		packet	:	pointer to 24 byte array to be cleared,
 *					update done in place
 *
 * Returns:
 *		 None
 *****************************************************************************/

void ClearPacketData(unsigned char *packet)
{
    int i;

	if (packet == NULL)	return;

    for (i=0; i<SUBCODE_DATA_SZ; i++)
    {
        ClearCDG(((SubCodePacket*)packet)->subCode.data.filler[i]);
    }
}

/******************************************************************************
 *
 * Initialize CDG Packet
 *
 *    Initialize CDG Packet to zero.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		packet	:	pointer to 24 byte array intialized in place,
 *					update done in place
 *
 * Returns:
 *		 None
 *****************************************************************************/

void PackInitialize(unsigned char *packet)
{
    memset((void*)packet, '\0', SC_PACKET_SZ);
}

/******************************************************************************
 *
 * Package a BorderPreset Command
 *
 *    Set the border of the screen to a particular color.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		color	:	index into color table,
 *					valid range is 0 to 15
 *		packet	:	pointer to 24 byte array to receive packed command,
 *					update done in place
 *
 * Returns:
 *		 0		:	successfull completion
 *		-1		:	undefined packet pointer
 *		-2		:	color table index out of range
 *****************************************************************************/

int PackBorderPreset(unsigned int color, unsigned char *packet)
{
	if (packet == NULL)					return(-1);
	if (color   > CDG_MAX_COLOR_INDEX)	return(-2);

    ClearPacketData(packet);

    PutCommand    (((SubCodePacket*)packet)->subCode);
    PutInstruction(((SubCodePacket*)packet)->subCode, BORDER_PRESET_SUBCODE);
    PutColor      (((SubCodePacket*)packet)->subCode.data.borderPreset.color, color);

	return(0);
}

/******************************************************************************
 *
 * Package a MemoryPreset Command
 *
 *    Set the screen to a particular color.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		color	:	index into color table,
 *					valid range is 0 to 15
 *		packet	:	pointer to 24 byte array to receive packed command,
 *					update done in place
 *
 * Returns:
 *		 0		:	successfull completion
 *		-1		:	undefined packet pointer
 *		-2		:	color table index out of range
 *****************************************************************************/

int PackMemoryPreset(unsigned int color,
                                   unsigned int sequenceNumber,
                                   unsigned char *packet)
{
	if (packet == NULL)					return(-1);
	if (color   > CDG_MAX_COLOR_INDEX)	return(-2);

    ClearPacketData(packet);

    PutCommand    (((SubCodePacket*)packet)->subCode);
    PutInstruction(((SubCodePacket*)packet)->subCode, MEMORY_PRESET_SUBCODE);
    PutColor      (((SubCodePacket*)packet)->subCode.data.memPreset.color, color);
    PutRepeat     (((SubCodePacket*)packet)->subCode.data.memPreset.repeat, sequenceNumber);

	return(0);
}


/******************************************************************************
 *
 * Package Tile Block Normal Command
 *
 *    Load a 12 x 6, 2 color tile and display by direct copying.
 *
 * Package Tile Block XOR Command
 *
 *    Load a 12 x 6, 2 color tile and display it using the XOR method.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		color0	 :	index into color table for 0 bits,
 *					valid range is 0 to 15
 *		color1	 :	index into color table for 1 bits,
 *					valid range is 0 to 15
 *      row      :  horizontal position of the tile in tile coordinate space,
 *                  valid range is 0 to 50
 *      column   :  vertical position of the tile in tile coordinate space,
 *                  valid range is 0 to 18
 *      bitmap[] :  pointer to 12 byte array containing the actual bit values
 *                  for the tile, six pixels per byte. The uppermost valid
 *                  bit of each byte (0x20) contains the left-most pixel of
 *                  each scanline of the tile
 *		packet[] :  pointer to 24 byte array to receive packed command,
 *					update done in place
 *
 * Returns:
 *		 0		 :  successfull completion
 *		-1		 :  undefined packet pointer
 *		-2		 :  color0 table index out of range
 *		-3		 :  color1 table index out of range
 *      -4       :  undefined bitmap pointer
 *      -5       :  row value out of range
 *      -6       :  column value out of range
 *****************************************************************************/

int PackTile(unsigned char  subcode,
             unsigned int   color0,
             unsigned int   color1,
             unsigned int   row,
             unsigned int   column,
             unsigned char *bitmap,
             unsigned char *packet)
{
    int i;

	if (packet == NULL)					return(-1);
	if (color0   > CDG_MAX_COLOR_INDEX)	return(-2);
	if (color1   > CDG_MAX_COLOR_INDEX)	return(-3);
	if (bitmap == NULL)					return(-4);

	if (row    < 0 || row    > MAX_SCREEN_ROW_TILES)    return(-5);
    if (column < 0 || column > MAX_SCREEN_COLUMN_TILES) return(-6);

    ClearPacketData(packet);

    PutCommand    (((SubCodePacket*)packet)->subCode);
    PutInstruction(((SubCodePacket*)packet)->subCode, subcode);
    PutColor      (((SubCodePacket*)packet)->subCode.data.tile.color0, color0);
    PutColor      (((SubCodePacket*)packet)->subCode.data.tile.color1, color1);
    PutRow        (((SubCodePacket*)packet)->subCode.data.tile.row,
                   (unsigned char)row);
    PutColumn     (((SubCodePacket*)packet)->subCode.data.tile.column,
                   (unsigned char)column);

    for (i=0; i<TILE_BLOCK_FILLER_SZ; i++)
        PutBitmapRow(((SubCodePacket*)packet)->subCode.data.tile.pixels[i],
                     bitmap[i]);

	return(0);
}

int PackTileNormal(unsigned int   color0,
                                 unsigned int   color1,
                                 unsigned int   row,
                                 unsigned int   column,
                                 unsigned char *bitmap,
                                 unsigned char *packet)
{
    return(PackTile(TILE_BLOCK_NORMAL_SUBCODE,
                    color0,
                    color1,
                    row,
                    column,
                    bitmap,
                    packet));
}

int PackTileXOR(unsigned int   color0,
                              unsigned int   color1,
                              unsigned int   row,
                              unsigned int   column,
                              unsigned char *bitmap,
                              unsigned char *packet)
{
    return(PackTile(TILE_BLOCK_XOR_SUBCODE,
                    color0,
                    color1,
                    row,
                    column,
                    bitmap,
                    packet));
}

/******************************************************************************
 *
 * 
 * Package Scroll Preset Command
 *
 *    Scroll the image, filling in the new area with a color.
 *
 * Package Scroll Copy Command
 *
 *    Scroll the image, rotating the bits back around.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		color	 :	index into color table for 0 bits,
 *					valid range is 0 to 15
 *      direction:	direction of scroll,
 *					valid range is one of SCROLL_UP, SCROLL_DOWN, SCROLL_LEFT,
 *					or SCROLL_RIGHT
 *      pixels   :	number of pixels to scroll,
 *					valid range for horizontal scroll is 0 to 6,
 *					valid range for vertical scroll is 0 to 12
 *		packet[] :  pointer to 24 byte array to receive packed command,
 *					update done in place
 *
 * Returns:
 *		 0		 :  successfull completion
 *		-1		 :  undefined packet pointer
 *		-2		 :  color table index out of range
 *      -3       :  pixel count out of range
 *      -4       :  invalid scrolling direction
 *****************************************************************************/

int PackScroll(unsigned int   subCode,
               unsigned int   color,
               Scroll_Control direction,
               unsigned int   pixels,
               unsigned char *packet)
{
    int scroll;

	if (packet == NULL)					return(-1);
	if (color   > CDG_MAX_COLOR_INDEX)	return(-2);

    ClearPacketData(packet);

    PutCommand    (((SubCodePacket*)packet)->subCode);
    PutInstruction(((SubCodePacket*)packet)->subCode, subCode);
    PutColor      (((SubCodePacket*)packet)->subCode.data.scroll.color, color);

    scroll = pixels;
    switch(direction)
    {
        case SCROLL_UP:
            if (pixels  > MAX_VERTICAL_SCROLL) return(-3);
            if (pixels == MAX_VERTICAL_SCROLL)
                scroll = MAX_VERTICAL_SCROLL_UP;

            PutScroll(((SubCodePacket*)packet)->subCode.data.scroll.vScroll, scroll);
           break;

        case SCROLL_DOWN:
            if (pixels  > MAX_VERTICAL_SCROLL) return(-3);
            if (pixels == MAX_VERTICAL_SCROLL)
                scroll = MAX_VERTICAL_SCROLL_DOWN;

            PutScroll(((SubCodePacket*)packet)->subCode.data.scroll.vScroll, scroll);
            break;

        case SCROLL_RIGHT:
            if (pixels  > MAX_HORIZONTAL_SCROLL) return(-3);
            if (pixels == MAX_HORIZONTAL_SCROLL)
                scroll = MAX_HORIZONTAL_SCROLL_RIGHT;

            PutScroll(((SubCodePacket*)packet)->subCode.data.scroll.hScroll, scroll);
            break;

        case SCROLL_LEFT: 
            if (pixels  > MAX_HORIZONTAL_SCROLL) return(-3);
            if (pixels == MAX_HORIZONTAL_SCROLL)
                scroll = MAX_HORIZONTAL_SCROLL_LEFT;

            PutScroll(((SubCodePacket*)packet)->subCode.data.scroll.hScroll, scroll);
            break;

        default:
            return(-4);
    }

	return(0);
}


int PackScrollCopy(Scroll_Control direction,
                                 unsigned int   pixels,
                                 unsigned char *packet)
{
    return (PackScroll(SCROLL_COPY_SUBCODE,
                       NO_COLOR,
                       direction,
                       pixels,
                       packet));
}


int PackScrollPreset(Scroll_Control direction,
                                   unsigned int   color,
                                   unsigned int   pixels,
                                   unsigned char *packet)
{
    return (PackScroll(SCROLL_PRESET_SUBCODE,
                       color,
                       direction,
                       pixels,
                       packet));
}

/******************************************************************************
 *
 * Package Define Trasparent Color Command
 *
 *    Indicate the specified color is to be treated as transparent.
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		color	:	index into color table,
 *					valid range is 0 to 15
 *		packet	:	pointer to 24 byte array to receive packed command,
 *					update done in place
 *
 * Returns:
 *		 0		:	successfull completion
 *		-1		:	undefined packet pointer
 *		-2		:	color table index out of range
 *****************************************************************************/

int PackTransperentColor(unsigned int   color,
                                       unsigned char *packet)
{
	if (packet == NULL)					return(-1);
	if (color   > CDG_MAX_COLOR_INDEX)	return(-2);

    ClearPacketData(packet);

    PutCommand    (((SubCodePacket*)packet)->subCode);
    PutInstruction(((SubCodePacket*)packet)->subCode, DEFINE_TRASPARENT_COLOR_SUBCODE);
    PutColor      (((SubCodePacket*)packet)->subCode.data.transColor, color);
    
	return(0);
}


/******************************************************************************
 *
 * Package Load Color Table Lo Command
 * Package Load Color Table High Command
 *
 *-----------------------------------------------------------------------------
 *
 * Parameters:
 *		colorTable	:  pointer to 16 byte array containing 8, 12 bit RGB values
 *		packet	    :  pointer to 24 byte array to receive packed command,
 *	                   update done in place
 *
 * Returns:
 *		 0		:	Successfull completion
 *		-1		:	undefined packet pointer
 *		-2		:	undefined colorTable pointer
 *****************************************************************************/

int PackColorTable(unsigned int    subCode,
                   unsigned short *colorTable,
                   unsigned char  *packet)
{
    int i, j, hcolor, lcolor;

	if (packet     == NULL)	return(-1);
	if (colorTable == NULL)	return(-2);

    ClearPacketData(packet);

    PutCommand    (((SubCodePacket*)packet)->subCode);
    PutInstruction(((SubCodePacket*)packet)->subCode, subCode);

    for(i=0, j=0; i<LOAD_COLOR_TABLE_COLOR_SZ; i++, j++)
    {
        lcolor =  colorTable[i] & LO_CLUT_MASK;
        hcolor = (colorTable[i] & HIGH_CLUT_MASK) >> HIGH_CLUT_OFFSET;
        PutCLUTColor (((SubCodePacket*)packet)->subCode.data.loadCLUT.color[j], hcolor);
        j++;
        PutCLUTColor (((SubCodePacket*)packet)->subCode.data.loadCLUT.color[j], lcolor);
    }

    return(0);
 }

int PackColorTableLo(unsigned short *colorTable,
                                   unsigned char  *packet)
{
    return(PackColorTable(LOAD_COLOR_TABLE_LO_SUBCODE, colorTable, packet));
}

int PackColorTableHigh(unsigned short *colorTable,
                                     unsigned char  *packet)
{
    return(PackColorTable(LOAD_COLOR_TABLE_HIGH_SUBCODE, colorTable, packet));
}
