/*  
 *
 *  (c) Computall Services 1999/2000
 *  url:  http://www.ncf.ca/~aa571/index.html
 *  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


// this software is produced as a sample piece of code to exercise the
// functions contained in cdg_lib.

// permission is granted for free use this demo software to develop 
// applications using cdg_lib as long as the applications are given away
// for free and are not sold.

#include <stdio.h>
// these are the functions written by Norm Bell
// permission is granted for free use this demo software to develop 
// applications as long as the applications are given away
// for free and are not sold.
#include "..\cdg_pack\cdgpacker.h"
#include "..\cdg_pack\cdgdef.h"


// the library is made up of functions in three categories
// 1) interleaving/deinterleaving
// 2) error correction
// 3) error coding

// INTERLEAVING
// there are two functions, cdg_interleave and cdg_deinterleave
// de-interleave one pack
// the function deinterleave takes a single pointer to a string of
// 24 bytes and returns the deinterleaved 24 bytes in the same place
extern int cdg_deinterleave(unsigned char *);
// interleave one pack 
// the function interleave takes a single pointer to a string of
// 24 bytes and returns the interleaved 24 bytes in the same place
extern int cdg_interleave(unsigned char *);

// ERROR CORRECTING
// error correct pack header
// the function cdg_rs_2_4_decode takes a single pointer to a string of
// 24 bytes and returns the 24 bytes in the same place with the 4 header
// bytes error corrected
extern int cdg_rs_2_4_decode(unsigned char *);
// error correct pack
// the function cdg_rs_2_4_decode takes a single pointer to a string of
// 24 bytes and returns the 24 bytes in the same place with the 24 pack
// bytes error corrected
extern int cdg_rs_20_24_decode(unsigned char *);
// the function temp_cdg_rs_20_24 is a temporary function that will return
// a 0 if the 4 bytes of the header of a pack are correct, and will return
// a 1 if they are not 
extern int temp_cdg_rs_2_4(unsigned char *);

//ERROR CODING
// generate rs codes for pack header
// the function cdg_rs_2_4_code takes a single pointer to a string of 
// 24 bytes and returns a pack with the 4 bytes of header error protected
extern int cdg_rs_2_4_code(unsigned char *);
// error correct pack
// the function cdg_rs_20_42_code takes a single pointer to a string of 
// 24 bytes and returns a pack with the 24 bytes error protected
extern int cdg_rs_20_24_code(unsigned char *);

// initialize the rs lut values
// the function cdg_rs_lut_init is a simple function that needs to be called
// the first time the dll is used so that it can initialize the rs tables
extern void cdg_rs_lut_init(void);

// DATA MERGING
// this is a routine that will do it's best to merge the raw rw
// subcode packs with the pcm audio data - first support is just
// wav files, but hopefully the xaudio guys will give us the ok
// to get to use their free mp3 decoder....
extern int cdg_rw_pcm_merge(void *, void *, void *);

// DATA BURNING
// if by some miracle we can find someone to port over enough of the
// cdrdao code, it would mean that we could also burn the cds with
// this program instead of having to use another program like goldenhawks
// package CDRWIN


// these are the two functions written by Heiko to generate the
// rs codes - the 2,4 encoder seems to work fine, but the 20,24
// encoder only seems to work with some packs, but not all....
extern int encode_LSUB_Q(unsigned char *);
extern int encode_LSUB_P(unsigned char *);




int  main (void)
{
unsigned char d[25];
unsigned char e[25];
unsigned char buffer[25];
FILE *in,*out;

// these are the variables needed for the sample code written
// by Norm Bell
int				i, j;
unsigned char	color = 15;
unsigned char	packet[24];
unsigned char bitmap[12] = {0x07, 0x07, 0x07, 0x07, 0x07, 0x07,
                               0x38, 0x38, 0x38, 0x38, 0x38, 0x38,};
unsigned short colorTable[16] = {0x0001, 0x000F, 0x0010, 0x001F,
                                     0x0101, 0x010F, 0x0110, 0x011F,
                                     0x00FF, 0x02FF, 0x04FF, 0x06FF,
                                     0x08FF, 0x0CFF, 0x0EFF, 0x0FFF};


// this is the start of the basic set of routines in cdg_lib
printf("CD+G Library test - version 0.1 - Copyright 1999/2000 Computall Services\n\n");

// before doing any calls into the cdg_lib, first call the
// lut init function
printf("initializing the reed solomon look up tables: ");
cdg_rs_lut_init();
printf("done\n");

// demonstrate the deinterleaving and error correcting
// use the file from a toshiba drive that has not been
// deinterleaved and not error corrected
if ((in = fopen("toshcdg.cdg", "rb"))== NULL)
	{
	printf("Cannot open toshcdg.cdg\n");
	return 1;
	}

if ((out = fopen("toshdein.cdg", "w+b" )) == NULL)
	{
	printf("Cannot open toshdein.cdg\n");
	return 1;
	}

printf("de-interleaving packs from file: ");

d[0] = fgetc(in) & 0x3f;
while (!feof(in))
	{
	// read in one pack
	for (i=1;i<24;i++)
		d[i] = fgetc(in) & 0x3f;

	// de-interleave one pack
	cdg_deinterleave(d);

	/*
	// leave this code commented out until we have this routine
	// just use the temporary rs_2_4 routine to check for errors
	// instead of being able to correct them. only errors in the 
	// header may be detected, and any errors in the remainder of
	// the pack will be NOT found and will be saved to the output
	// file unfixed

	// error correct pack header
	cdg_rs_2_4_decode(d);
	// error correct pack
	cdg_rs_20_24_decode(d);
	*/

	if (temp_cdg_rs_2_4(d)==0)
		for (i=0;i<24;i++)
			buffer[i] = d[i];
	else
		for (i=0;i<24;i++)
			buffer[i] = 0;
	// output one pack
	fwrite(buffer, 1, 24, out);

	// get the first byte of next pack
	d[0] = fgetc(in) & 0x3f;
	} // end while

fclose(in);
fclose(out);
printf("done\n");

// now demonstrate the error coding and interleaving
// use the sample cdg file grabbed from a plextor drive
// so it is already been error corrected so it should be
// clean data

if ((in = fopen("plexcdg.cdg", "rb"))== NULL)
	{
	printf("Cannot open plexcdg.cdg\n");
	return 1;
	}

if ((out = fopen("cdgerint.cdg", "w+b" )) == NULL)
	{
	printf("Cannot open cdgerint.cdg\n");
	return 1;
	}

printf("interleaving packs from file: ");
d[0] = fgetc(in) & 0x3f;
while (!feof(in))
	{
// read in one pack
	for (i=1;i<24;i++)
		{
		d[i] = fgetc(in) & 0x3f;
		e[i] = d[i];
		}

// null out the rs codes to show the functions work
	d[2] = 0;
	d[3] = 0;
	d[20] = 0;
	d[21] = 0;
	d[22] = 0;
	d[23] = 0;

	// error correct Q codes
	encode_LSUB_Q(d);
	if ((d[2]==e[2]) &&
		(d[3]==e[3]))
		{
		//printf("encoding matches\n");
		}
	else
		{
		printf("q encoding doesn't match: ");
		printf("%.2x %.2x",e[2],e[3]);
		printf(" - %.2x %.2x\n",d[2],d[3]);
		}


	// error correct P codes
	encode_LSUB_P(d);
	if ((d[20]==e[20]) &&
		(d[21]==e[21]) &&
		(d[22]==e[22]) &&
		(d[23]==e[23]))
		{
//		printf("p encoding matches: ");
//		printf("%.2x %.2x %.2x %.2x",e[20],e[21],e[22],e[23]);
//		printf(" - %.2x %.2x %.2x %.2x\n",d[20],d[21],d[22],d[23]);
		}
	else
		{
//		printf("p encoding doesn't match: ");
//		printf("%.2x %.2x %.2x %.2x",e[20],e[21],e[22],e[23]);
//		printf(" - %.2x %.2x %.2x %.2x\n",d[20],d[21],d[22],d[23]);
		}
	/*
	// leave this code commented out until we have this routine
	// error correct pack
	cdg_rs_20_24_code(d);
	*/

	// interleave one pack
	cdg_interleave(d);

	// write pack to file
	fwrite(d, 1, 24, out);

	// get first byte of new pack
	d[0] = fgetc(in) & 0x3f;
	} // end while

fclose(in);
fclose(out);
printf("done\n");

// start of example code from Norm Bell	
	// initialize a single packet   
    PackInitialize(packet);

	// sample of how to do a border preset command (2)
	if (PackBorderPreset(color, packet) == 0)
		{
		printf("PackBorderPreset Worked!\n");
		for (i=0; i<10; i++) printf("[%d] = %d\n",i, packet[i]);
		}
	else
		{
		printf("Failed!\n");
		}
	
	// sample of how to do a memory preset command (1)
	for (j=0; j<5; j++)
	    {
    	if (PackMemoryPreset(8, j, packet) == 0)
		    {
		    printf("PackMemoryPreset Worked!\n");
		    for (i=0; i<10; i++) printf("[%d] = %d\n",i, packet[i]);
		    }
	    else
		    {
    		printf("Failed!\n");
			}
	    }

	// how to do a tile command 
	// is the way of putting a 6x12 pixel block on the screen
    if (PackTileNormal(5, 6, 1, 1, bitmap, packet) == 0)
		{
		printf("PackTileNormal Worked!\n");
		for (i=0; i<24; i++) printf("[%d] = %d\n",i, packet[i]);
		}
	else
		{
		printf("Failed!\n");
		}

	// how to do a exclusive-or tile command
    if (PackTileXOR   (7, 8, 2, 2, bitmap, packet) == 0)
		{
		printf("PackTileXOR Worked!\n");
		for (i=0; i<24; i++) printf("[%d] = %d\n",i, packet[i]);
		}
	else
		{
		printf("Failed!\n");
		}

	// how to do a define transparency colour command (28)
    if (PackTransperentColor(9, packet) == 0)
		{
		printf("PackTransperentColor Worked!\n");
		for (i=0; i<10; i++) printf("[%d] = %d\n",i, packet[i]);
		}
	else
		{
		printf("Failed!\n");
		}

	// how to do a scroll up command (12)
	// is a copy up with no horizontal copy
    if (PackScrollCopy(SCROLL_UP, 7, packet) == 0)
		{
		printf("PackScrollCopy Worked!\n");
		for (i=0; i<10; i++) printf("[%d] = %d\n",i, packet[i]);
		}
	else
		{
		printf("Failed!\n");
		}

	// how to do a scroll down command (12)
	// is a copy down with no horizontal copy
    if (PackScrollPreset(SCROLL_DOWN, color, 12, packet) == 0)
		{
		printf("PackScrollPreset Worked!\n");
		for (i=0; i<10; i++) printf("[%d] = %d\n",i, packet[i]);
		}
	else
		{
		printf("Failed!\n");
		}

	// how to do a scroll left command (12)
	// is a copy left with no vertical copy
    if (PackScrollCopy(SCROLL_LEFT, 6, packet) == 0)
		{
		printf("PackScrollCopy Worked!\n");
		for (i=0; i<10; i++) printf("[%d] = %d\n",i, packet[i]);
		}
	else
		{
		printf("Failed!\n");
		}

	// how to do a scroll right command (12)
	// is a copy right with no vertical copy
    if (PackScrollPreset(SCROLL_RIGHT, color, 5, packet) == 0)
		{
		printf("PackScrollPreset Worked!\n");
		for (i=0; i<10; i++) printf("[%d] = %d\n",i, packet[i]);
		}
	else
		{
		printf("Failed!\n");
		}

	// how to do a load colour table (low half)
    if (PackColorTableLo(colorTable, packet) == 0)
		{
		printf("PackColorTableLo Worked!\n");
		for (i=0; i<24; i++) printf("[%d] = %d\n",i, packet[i]);
		}
	else
		{
		printf("Failed!\n");
		}

	// how to do a load colour table (high half)
    if (PackColorTableHigh(&colorTable[8], packet) == 0)
		{
		printf("PackColorTableHigh Worked!\n");
		for (i=0; i<24; i++) printf("[%d] = %d\n",i, packet[i]);
		}
	else
		{
		printf("Failed!\n");
		}
// end of example code from Norm Bell



return 0;
}
